SystemOrganization addCategory: #'OmniST-Languages-LRSTM-Errors'!
SystemOrganization addCategory: #'OmniST-Languages-LRSTM-Model'!
SystemOrganization addCategory: #'OmniST-Languages-POCs'!
SystemOrganization addCategory: #'OmniST-Languages-AmbientTalk'!
SystemOrganization addCategory: #'OmniST-Languages-AmbientTalk-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-ActiveObject'!
SystemOrganization addCategory: #'OmniST-Languages-ActiveObject-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-Clojure'!
SystemOrganization addCategory: #'OmniST-Languages-Clojure-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-CSPi-Reflection'!
SystemOrganization addCategory: #'OmniST-Languages-CSPi'!

OstDomain subclass: #ATActor
	instanceVariableNames: 'process mailbox terminate name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk-Reflection'!

!ATActor commentStamp: 'StefanMarr 4/22/2012 20:01' prior: 0!
The main idea behind the implementation of ATActors is that the Smalltalk semantics already provides encapsulation on object level.
Thus, we can rely on redefining the requestExection handlers, but do not need to implement anything else.
This provides us the whole actor semantics since the requestExection handler will turn calls into async requests.


TODO: literals/globals are still a problematic loop hole. We do not consider them at the moment. oops...
We probably should make a local copy of first request and then use that exclusively, or something like that. But here we are really fighting against our Smalltalk heritage, and well, we could make it safe...!

!ATActor class methodsFor: 'VM support' stamp: 'StefanMarr 7/29/2012 23:12'!
domainCustomizations ^ 16rFFF000! !

!ATActor class methodsFor: 'constructor' stamp: 'StefanMarr 3/22/2012 16:59'!
newWithInitializer: aBlock
	"Normally, we would return a reference to the domain,
	 but this is not really the way AmbientTalk works,
	 so, here we also return the value returned by the aBlock."
	| domain valueMsg promise |
	domain := self new.
	promise := domain evaluateAsync: [aBlock omni: domain].
	^ promise read.! !

!ATActor methodsFor: 'actor semantics' stamp: 'StefanMarr 3/31/2012 22:25'!
evaluateAsync: aBlock
	"This is an implementation level method.
	 The block is by convention not referring to
	 unsafe shared state."
	| msg |
	msg := ATMessage new.
	msg receiver: aBlock.
	msg setSelector: #value arguments: #().

	mailbox nextPut: msg.
	
	^ msg promise.! !

!ATActor methodsFor: 'initialization' stamp: 'StefanMarr 1/4/2012 13:58'!
initialize
	super initialize.
	
	mailbox := SharedQueue new.
	terminate := false.
	
	process := self spawnHere: [
		[terminate] whileFalse: [
			self processIncomingMessages.
		]].
	process name: 'Actor'.! !

!ATActor methodsFor: 'debugging' stamp: 'StefanMarr 1/4/2012 13:43'!
name
	^ name! !

!ATActor methodsFor: 'debugging' stamp: 'StefanMarr 1/4/2012 13:58'!
name: aString
	name := aString.
	
	process ifNotNil: [ process name: 'Actor [', aString, ']' ].! !

!ATActor methodsFor: 'printing' stamp: 'StefanMarr 1/4/2012 13:54'!
printOn: aStream
	super printOn: aStream.
	
	name ifNotNil: [
		aStream nextPutAll: '[', name, ']'.].! !

!ATActor methodsFor: 'actor semantics' stamp: 'StefanMarr 7/19/2012 22:47'!
processIncomingMessages
	| msg result |
	<omniUnenforced>
	msg := mailbox next.
	
	result := msg send.
	
	msg result: result.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:04'!
requestExecution: p1 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: {p1}.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:05'!
requestExecution: p1 with: p2 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:05'!
requestExecution: p1 with: p2 with: p3 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:05'!
requestExecution: p1 with: p2 with: p3 with: p4 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3 with: p4.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:06'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:06'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:06'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:33'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7. p8 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:33'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7. p8. p9 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:06'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10 of: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: { p1. p2. p3. p4. p5. p6. p7. p8. p9. p10 }.! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/19/2012 22:49'!
requestExecutionOf: aSelector on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerform: aSelector.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: #().! !

!ATActor methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:35'!
requestExecutionOf: aSelector with: args lookup: cls on: anObj
	<omniUnenforced>
	"SEE #requestExecutionOf:on:with: for the template implementation"
	| msg |
	self == OstDomain currentDomain ifTrue: [
		^ anObj baselevelPerformSuper: aSelector with: args inLookupClass: cls.
	].
	
	^ self sendAsAsyncMessage: aSelector to: anObj with: args inLookupClass: cls.! !

!ATActor methodsFor: 'actor semantics' stamp: 'StefanMarr 1/3/2012 14:44'!
sendAsAsyncMessage: aSelector to: anObj with: anArgArray
	| msg |
	msg := ATMessage new.
	msg setSelector: aSelector arguments: anArgArray.
	msg receiver: anObj.
	mailbox nextPut: msg.
	
	^ msg result.! !

!ATActor methodsFor: 'actor semantics' stamp: 'StefanMarr 7/22/2012 02:14'!
terminate
	<omniUnenforced>
	self evaluateAsync: [
		terminate := true
	].! !

OstDomain subclass: #CSPiProcess
	instanceVariableNames: 'process state completionSema'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-CSPi'!

!CSPiProcess class methodsFor: 'VM support' stamp: 'StefanMarr 7/29/2012 23:12'!
domainCustomizations ^ 16rFFF000! !

!CSPiProcess methodsFor: 'initialization' stamp: 'StefanMarr 7/28/2012 10:59'!
initialize
	super initialize.
	completionSema := Semaphore new.
	state := #initialized.! !

!CSPiProcess methodsFor: 'process' stamp: 'StefanMarr 1/9/2012 23:46'!
join
	"Wait for completion"
	completionSema wait.! !

!CSPiProcess methodsFor: 'process' stamp: 'StefanMarr 1/9/2012 23:54'!
raiseIsolationViolationError
	Error signal: 'Entity is isolated. Access violation detected.'! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:04'!
requestExecution: p1 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:05'!
requestExecution: p1 with: p2 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:37'!
requestExecution: p1 with: p2 with: p3 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:37'!
requestExecution: p1 with: p2 with: p3 with: p4 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:37'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:37'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:37'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:37'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:38'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:38'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10 of: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/19/2012 22:49'!
requestExecutionOf: aSelector on: anObj
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObj perform: aSelector.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:40'!
requestExecutionOf: aSelector with: argArray lookup: cls on: anObject
	<omniUnenforced>
	self == OstDomain currentDomain ifTrue: [
		^ anObject perform: aSelector withArguments: argArray inSuperclass: cls.
	].
	
	^ self raiseIsolationViolationError! !

!CSPiProcess methodsFor: 'process runnable' stamp: 'StefanMarr 1/9/2012 23:18'!
run
	self subclassResponsibility! !

!CSPiProcess methodsFor: 'process' stamp: 'StefanMarr 7/18/2012 22:08'!
start
	<omniUnenforced>
	process := self spawnHere: [
		state := #running.
		[self run] omni: self.
		state := #done.
		completionSema signal.
	].
	process name: 'CSPi'.! !

OstDomain subclass: #LRSTMDomain
	instanceVariableNames: ''
	classVariableNames: 'Domain'
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMDomain class methodsFor: 'singleton' stamp: 'StefanMarr 12/11/2011 22:19'!
domain
	"We only need one domain for the STM system."
	Domain ifNil: [ Domain := self new. ].
	^ Domain! !

!LRSTMDomain class methodsFor: 'VM support' stamp: 'StefanMarr 8/5/2012 20:55'!
domainCustomizations ^ 16rF000FFF! !

!LRSTMDomain class methodsFor: 'initialization' stamp: 'StefanMarr 7/31/2012 22:43'!
initialize
	Domain := nil.! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:02'!
primAt: idx on: anObj
	<omniUnenforced>
	^ anObj workingCopy at: idx! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:02'!
primAt: idx on: anObj put: anObject
	<omniUnenforced>
	^ anObj workingCopy at: idx put: anObject! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:03'!
primBasicAt: idx on: anObj
	<omniUnenforced>
	^ anObj workingCopy basicAt: idx! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:03'!
primBasicAt: idx on: anObj put: anObject
	<omniUnenforced>
	^ anObj workingCopy basicAt: idx put: anObject! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:03'!
primInstVarAt: idx on: obj
	<omniUnenforced>
	^ obj workingCopy instVarAt: idx! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:03'!
primInstVarAt: idx on: obj put: aVal
	<omniUnenforced>
	^ obj workingCopy instVarAt: idx put: aVal! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 8/2/2012 10:59'!
primNext: obj
	<omniUnenforced>
	^ [ obj atomicNext ] omni: self! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 8/2/2012 10:59'!
primNext: obj put: val
	<omniUnenforced>
	^ [ obj atomicNextPut: val ] omni: self! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 8/1/2012 22:48'!
primReplaceFrom: start to: stop with: replacement startingAt: repStart on: obj
	<omniUnenforced>
	"REM: we need to use obj here directly, since we go back to the enforced execution in which the working copy would not exist.
	 The working copy is strictly a meta object."
	^ [ obj atomicReplaceFrom: start to: stop with: replacement startingAt: repStart ] omni: self.! !

!LRSTMDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:03'!
primShallowCopy: obj
	<omniUnenforced>
	^ obj workingCopy shallowCopy! !

!LRSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:04'!
readField: idx of: obj
	<omniUnenforced>
	^ obj workingCopy instVarAt: idx.! !

!LRSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:04'!
readLiteral: association
	<omniUnenforced>
	^ association workingCopy value! !

!LRSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 8/2/2012 20:09'!
write: val toField: idx of: obj
	<omniUnenforced>
	obj workingCopy instVarAt: idx put: val.
	^ val! !

!LRSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 8/2/2012 18:35'!
write: val toField: idx of: obj return: returnValue
	<omniUnenforced>
	obj workingCopy instVarAt: idx put: val.
	^ returnValue! !

!LRSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 8/5/2012 20:53'!
write: val toLiteral: association
	<omniUnenforced>
	association workingCopy value: val.
	^ val! !

!LRSTMDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 8/5/2012 20:53'!
write: val toLiteral: association return: returnValue
	<omniUnenforced>
	association workingCopy value: val.
	^ returnValue! !

OstDomain subclass: #OActiveObjectDomain
	instanceVariableNames: 'obj'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject-Reflection'!

!OActiveObjectDomain class methodsFor: 'VM support' stamp: 'StefanMarr 7/29/2012 23:12'!
domainCustomizations ^ 16rFFF000! !

!OActiveObjectDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/9/2012 17:17'!
newFor: anActiveObject
	| domain |
	domain := self new.
	domain object: anActiveObject.
	^ domain.! !

!OActiveObjectDomain methodsFor: 'omni-domain' stamp: 'StefanMarr 11/6/2011 21:04'!
domainForNewObjects
	"Active objects are created explicitly,
	 classes tell whether their objects are supposed to be active.
	 This happens during #initialize.
	 
	 However, #domainForNewObjects is determined during
	 #basicNew, and for ActiveObjects we just assume as the standard
	 case that all objects are passive.
	 For the moment that means, they are not part of any domain."
	^ nil! !

!OActiveObjectDomain methodsFor: 'accessing' stamp: 'StefanMarr 11/5/2011 20:27'!
object: anObject
	obj := anObject! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:04'!
requestExecution: p1 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:05'!
requestExecution: p1 with: p2 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:05'!
requestExecution: p1 with: p2 with: p3 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:47'!
requestExecution: p1 with: p2 with: p3 with: p4 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3. p4 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:47'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3. p4. p5 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:47'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3. p4. p5. p6 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:48'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3. p4. p5. p6. p7 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:48'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3. p4. p5. p6. p7. p8 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:48'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3. p4. p5. p6. p7. p8. p9 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:48'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10 of: aSelector on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10.
	].
	^ obj sendAsAsyncMessage: aSelector to: anObject with: { p1. p2. p3. p4. p5. p6. p7. p8. p9. p10 }
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/19/2012 22:49'!
requestExecutionOf: aSelector on: anObject
	<omniUnenforced>
	| normSel |
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector.
	].
	
	normSel := aSelector asNormalSelector.
	normSel = #processIncomingMessages ifTrue: [
		Error signal: '#processIncomingMessages is protected and cannot be called directly.'.
	].
	
	^ obj sendAsAsyncMessage: aSelector to: anObject with: #()
	
	! !

!OActiveObjectDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/20/2012 23:50'!
requestExecutionOf: aSelector with: argArray lookup: cls on: anObject
	<omniUnenforced>
	(obj == anObject and: [OstDomain currentDomain ~~ self]) ifFalse: [
		^ anObject perform: aSelector withArguments: argArray inSuperclass: cls.
	].
	
	^ obj sendAsAsyncMessage: aSelector to: anObject with: argArray
	
	! !

OstDomain subclass: #OPAtomDomain
	instanceVariableNames: 'obj mtx'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure-Reflection'!

!OPAtomDomain class methodsFor: 'VM support' stamp: 'StefanMarr 8/2/2012 20:53'!
domainCustomizations ^ 16r3001400! !

!OPAtomDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 01:42'!
newFor: anObject
	| domain |
	domain := self new.
	domain object: anObject.
	^ domain.! !

!OPAtomDomain methodsFor: 'omni-domain' stamp: 'StefanMarr 11/6/2011 21:46'!
domainForNewObjects
	"If I find a way to have these objects in an immutable domain, they should go there.
	 I could keep track of a epoch counter and based on that decide that they are modifiable during
	 the same epoch in which they were created!!
	TODO look into this"
	^ nil! !

!OPAtomDomain methodsFor: 'initialize-release' stamp: 'StefanMarr 7/28/2012 10:59'!
initialize
	super initialize.
	mtx := Mutex new.! !

!OPAtomDomain methodsFor: 'accessing' stamp: 'StefanMarr 11/6/2011 01:42'!
object: o
	obj := o! !

!OPAtomDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 11/6/2011 01:30'!
readField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here.
	 Will just read the data and return it."
	^ obj instVarAt: idx! !

!OPAtomDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/20/2012 17:56'!
requestExecution: aCompiledMethod of: anObj on: arguments
	"For the active object, we want to execute the message asynchronously.
	 But since we can't tell whether something is a void method,
	 we will always need to block on the promise and wait until
	 it is resolved."
	| requiresMtx sel |
	Transcript log: 'requestExecutionOf:'.
	
	sel := aCompiledMethod selector.
	
	requiresMtx :=  (sel = #compareTo:andSet:) or: [sel = #reset:].
	
	requiresMtx
		ifTrue:  [ mtx critical: [
			^ aCompiledMethod valueWithReceiver: obj arguments: arguments.]]
		ifFalse: [
			^ aCompiledMethod valueWithReceiver: obj arguments: arguments.].! !

!OPAtomDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 11/6/2011 01:46'!
write: val toField: idx of: ignoredObjUseKnownOwnerInstead
	"No strong encapsulation here, will just set the field and return the value."
	obj instVarAt: idx put: val.
	^ val! !

!OPAtomDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 8/2/2012 18:36'!
write: val toField: idx of: ignoredObjUseKnownOwnerInstead return: returnValue
	"No strong encapsulation here, will just set the field and return the value."
	obj instVarAt: idx put: val.
	^ returnValue! !

OstDomain subclass: #OcjAgentDomain
	instanceVariableNames: 'agent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure-Reflection'!

!OcjAgentDomain class methodsFor: 'VM support' stamp: 'StefanMarr 7/29/2012 23:12'!
domainCustomizations ^ 16r403000! !

!OcjAgentDomain class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/6/2012 13:09'!
newFor: anAgent
	| domain |
	domain := self new.
	domain agent: anAgent.
	^ domain.! !

!OcjAgentDomain methodsFor: 'accessing' stamp: 'StefanMarr 1/5/2012 14:42'!
agent: anAgent
	agent := anAgent! !

!OcjAgentDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:04'!
requestExecution: p1 of: aSelector on: anObject
	"An agent supports one synchronous operations with one parameters:
	  #send:"
	<omniUnenforced>
	| normSel |	
	anObject = agent ifFalse: [
		^ anObject perform: aSelector with: p1.].

	
	normSel := aSelector asNormalSelector.
	
	(normSel = #send:) ifTrue: [
		^ agent send: p1].
	
	"For everything else, we just assume someone added an explicit protocal to a Smalltalk subclass.
	 Thus, we follow Agent semantics by doing it asynchronously, but we do not give a return value.
	 Could be done with a promise, but we will encorage here asynchnous/agent-like programming."
	
	^ agent perform: #send: with: [:oldState | agent perform: aSelector with: p1.
														oldState].! !

!OcjAgentDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:05'!
requestExecution: p1 with: p2 of: aSelector on: anObject
	"An agent supports one synchronous operations with two parameters:
	  #send:with:"
	<omniUnenforced>
	| normSel |
	anObject = agent ifFalse: [
		^ anObject perform: aSelector with: p1 with: p2.].
	
	normSel := aSelector asNormalSelector.
	
	(normSel = #send:with:) ifTrue: [
		^ agent send: p1 with: p2].
	
	"For everything else, we just assume someone added an explicit protocal to a Smalltalk subclass.
	 Thus, we follow Agent semantics by doing it asynchronously, but we do not give a return value.
	 Could be done with a promise, but we will encorage here asynchnous/agent-like programming."
	
	^ agent perform: #send:with: with: [:oldState | agent perform: aSelector with: p1 with: p2.
															  oldState].! !

!OcjAgentDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/19/2012 22:49'!
requestExecutionOf: aSelector on: anObject
	"An agent supports three synchronous operations with no parameters:
	  #await
	  #read
	  #shutdown"
	<omniUnenforced>
	| normSel |
	
	"Rules are only enforced on the agent itself"
	anObject = agent ifFalse: [
		^ anObject perform: aSelector].
	
	normSel := aSelector asNormalSelector.
	
	(normSel = #read or:   [
	 normSel = #await or:  [
	 normSel = #shutdown   ]])
		ifTrue: [
		^ agent perform: normSel	].
	
	"There are two more selectors, we do not want to have someone messing with:"
	(normSel = #processIncomingMessages or: [
	 normSel = #initialize])
		ifTrue: [
			Error signal: 'Access to implementation methods is denied.'].
	
	"For everything else, we just assume someone added an explicit protocal to a Smalltalk subclass.
	 Thus, we follow Agent semantics by doing it asynchronously, but we do not give a return value.
	 Could be done with a promise, but we will encorage here asynchnous/agent-like programming."
	^ agent perform: #send: with: [:oldState | agent perform: aSelector.
														oldState].! !

OstDomain subclass: #OcjImmutableDomain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure-Reflection'!

!OcjImmutableDomain class methodsFor: 'VM support' stamp: 'StefanMarr 8/5/2012 20:55'!
domainCustomizations ^ 16rF00002A! !

!OcjImmutableDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:02'!
primAt: idx on: anObj put: aVal
	<omniUnenforced>
	self raiseImmutabilityError.! !

!OcjImmutableDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:03'!
primBasicAt: idx on: anObj put: aVal
	<omniUnenforced>
	self raiseImmutabilityError.! !

!OcjImmutableDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 7/18/2012 22:03'!
primInstVarAt: idx on: anObj put: aVal
	<omniUnenforced>
	self raiseImmutabilityError.! !

!OcjImmutableDomain methodsFor: 'immutability' stamp: 'StefanMarr 1/6/2012 15:14'!
raiseImmutabilityError
	Error signal: 'Attempt to modify the object failed. The owning domain does not allow mutations.'! !

!OcjImmutableDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:07'!
write: val toField: idx of: obj
	<omniUnenforced>
	self raiseImmutabilityError.! !

!OcjImmutableDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 8/2/2012 18:36'!
write: val toField: idx of: obj return: returnValue
	<omniUnenforced>
	self raiseImmutabilityError.
	^ returnValue! !

!OcjImmutableDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 7/18/2012 22:07'!
write: val toLiteral: association
	<omniUnenforced>
	self raiseImmutabilityError.! !

!OcjImmutableDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 8/5/2012 20:54'!
write: val toLiteral: association return: returnValue
	<omniUnenforced>
	self raiseImmutabilityError.
	^ returnValue! !

!ContextPart methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/31/2007 15:49'!
workingCopy
	^ self! !

Message subclass: #ATMessage
	instanceVariableNames: 'promise receiverObj'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk-Reflection'!

!ATMessage class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
method: aCompiledMethod on: anObj with: arguments
	| msg |
	msg := self new.
	msg method: aCompiledMethod.
	msg receiver: anObj.
	msg arguments: arguments.
	^ msg! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/4/2012 14:11'!
initialize
	super initialize.
	promise := OPromise new.! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:53'!
promise
	^ promise! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
receiver
	^ receiverObj! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
receiver: anObj
	receiverObj := anObj! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/2/2012 16:52'!
result
	"Result is blocking on the promise.
	 Use #promise to obtain the promise directly."
	^ promise read! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 22:52'!
result: anObj
	promise deliver: anObj! !

!ATMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/23/2012 13:30'!
send
	"answer the result of sending this message to its receiver"

	^ receiverObj baselevelPerform: selector withArguments: args! !

Message subclass: #OActiveObjectMessage
	instanceVariableNames: 'promise receiverObj'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject-Reflection'!

!OActiveObjectMessage class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/5/2011 17:17'!
method: aCompiledMethod on: anObj with: arguments
	| msg |
	msg := self new.
	msg method: aCompiledMethod.
	msg receiver: anObj.
	msg arguments: arguments.
	^ msg! !

!OActiveObjectMessage methodsFor: 'initialization' stamp: 'StefanMarr 11/4/2011 17:23'!
initialize
	super initialize.
	promise := OPromise new.! !

!OActiveObjectMessage methodsFor: 'accessing' stamp: 'StefanMarr 11/5/2011 23:17'!
receiver
	^ receiverObj! !

!OActiveObjectMessage methodsFor: 'accessing' stamp: 'StefanMarr 11/5/2011 17:19'!
receiver: anObj
	receiverObj := anObj! !

!OActiveObjectMessage methodsFor: 'promise' stamp: 'StefanMarr 11/4/2011 17:24'!
resolve
	^ promise read.! !

!OActiveObjectMessage methodsFor: 'accessing' stamp: 'StefanMarr 1/9/2012 17:48'!
result
	^ promise! !

!OActiveObjectMessage methodsFor: 'accessing' stamp: 'StefanMarr 11/5/2011 23:19'!
result: anObj
	promise deliver: anObj! !

!OActiveObjectMessage methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/9/2012 18:41'!
send
	"answer the result of sending this message to its receiver"
	^ receiverObj perform: selector withArguments: args! !

!WriteStream methodsFor: '*OmniST-Languages-LRSTM' stamp: 'StefanMarr 12/4/2011 22:17'!
atomicNextPut: anObject
	((collection class == ByteString) and: [
		anObject isCharacter and:[anObject isOctetCharacter not]]) ifTrue: [
			collection := (WideString from: collection).
			^self nextPut: anObject.
	].
	position >= writeLimit
		ifTrue: [^ self pastEndPut: anObject]
		ifFalse: 
			[position := position + 1.
			^collection at: position put: anObject]! !

OstObject subclass: #ATHelloWorld
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-AmbientTalk'!

!ATHelloWorld class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/7/2012 23:30'!
example
	| hello |
	hello := ATHelloWorld new.
	hello sayHello.
	hello saySomething: 'baz'.
	
	hello := ATActor newWith: [ ATHelloWorld new ].
	hello sayHello.
	hello saySomething: 'bar'.
! !

!ATHelloWorld methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/22/2012 14:39'!
sayHello
	Transcript log: 'Hello World from within an actor. Actor Domain:',
						(OstMirror domainOf: self) printString, '(', (OstMirror domainOf: self) hash asString, ')'.
	self saySomething: 'foooo'.! !

!ATHelloWorld methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/22/2012 14:43'!
saySomething: aString
	Transcript log: 'Something: ', aString, '. Actor Domain:', 
						(OstMirror domainOf: self) printString, '(', (OstMirror domainOf: self) hash asString, ')'.! !

OstObject subclass: #OActiveObject
	instanceVariableNames: 'mailbox terminate process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-ActiveObject'!

!OActiveObject commentStamp: 'StefanMarr 1/9/2012 16:12' prior: 0!
An active object enforces that all operations on it
are executed under its own control, i.e., in its own thread(Process).

All non-active objects are call passive objects and do not impose any
restrictions. Thus, messages are processes in the context of the sending
active/passive object. Passive objects do not have their own thread of control.

All non-active objects are passive objects, i.e., only objects inheriting from
ActiveObject are to be considered active objects.
!

!OActiveObject methodsFor: 'initialization' stamp: 'StefanMarr 7/18/2012 21:57'!
initialize
	<omniUnenforced>
	| domain |
	super initialize.
	
	mailbox := SharedQueue new.
	terminate := false.
	
	domain := OActiveObjectDomain newFor: self.
	OstMirror setDomainOf: self to: domain.
	
	process := domain newProcessFor: [
		[terminate and: [mailbox isEmpty]] whileFalse: [
			self processIncomingMessages.
		]].
	process name: 'OActiveObject'.
	process resume.! !

!OActiveObject methodsFor: 'as yet unclassified' stamp: 'StefanMarr 7/18/2012 22:03'!
processIncomingMessages
	<omniUnenforced>
	| msg result |
	msg := mailbox next.
	
	result := msg send.
	
	msg result: result.! !

!OActiveObject methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/9/2012 18:36'!
sendAsAsyncMessage: aSelector to: anObj with: anArgArray
	| msg |
	msg := OActiveObjectMessage new.
	msg setSelector: aSelector arguments: anArgArray.
	msg receiver: anObj.
	
	mailbox nextPut: msg.
	
	^ msg result.! !

OstObject subclass: #OcjAgent
	instanceVariableNames: 'state mailbox terminate process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure'!

!OcjAgent class methodsFor: 'instance creation' stamp: 'StefanMarr 7/18/2012 22:01'!
new: initialState
	"Work around omni enforcement"
	<omniUnenforced>
	
	| agent stateIdx |
	agent := self new.
	agent instVarNamed: #state put: initialState.
	
	^ agent.! !

!OcjAgent methodsFor: 'agent' stamp: 'StefanMarr 7/18/2012 21:56'!
await
	"Block until all actions have been processed (weaker semantics than Clojure's)"
	<omniUnenforced>
	[mailbox isEmpty] 
		whileFalse: [
			(Delay forMilliseconds: 50) wait.
		]! !

!OcjAgent methodsFor: 'initialization' stamp: 'StefanMarr 7/18/2012 21:57'!
initialize
	<omniUnenforced>
	| domain |
	super initialize.
	
	mailbox := SharedQueue new.
	terminate := false.
	
	domain := OcjAgentDomain newFor: self.
	OstMirror setDomainOf: self to: domain.
	
	process := domain newProcessFor: [
		[terminate and: [mailbox isEmpty]] whileFalse: [
			self processIncomingMessages.
		]].
	process name: 'OcjAgent'.
	process resume.! !

!OcjAgent methodsFor: 'agent' stamp: 'StefanMarr 1/7/2012 12:38'!
processIncomingMessages
	| updateBlock |
	updateBlock := mailbox peekWaiting. "Wait until we have it."

	state := updateBlock value: state.
	
	mailbox next. "Removing it only now avoids races with await"! !

!OcjAgent methodsFor: 'agent' stamp: 'StefanMarr 7/18/2012 22:03'!
read
	<omniUnenforced>
	^ state! !

!OcjAgent methodsFor: 'agent' stamp: 'StefanMarr 7/18/2012 22:08'!
send: anUpdateBlock
	<omniUnenforced>
	mailbox nextPut: anUpdateBlock.! !

!OcjAgent methodsFor: 'agent' stamp: 'StefanMarr 7/18/2012 22:08'!
send: anUpdateBlock with: args
	<omniUnenforced>
	self send: [:old | anUpdateBlock value: old value: args ]! !

!OcjAgent methodsFor: 'agent' stamp: 'StefanMarr 7/18/2012 22:08'!
shutdown
	"Make sure that the process loop gets around to see that terminate is set to true.
	 So, we just set it ourselves.
	 No race conditions here, purely sequential."
	<omniUnenforced>
	mailbox nextPut: [:ignore | terminate := true].! !

OcjAgent subclass: #OcjAgentWithImmutability
	instanceVariableNames: 'immutabilityDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure'!

!OcjAgentWithImmutability methodsFor: 'initialization' stamp: 'StefanMarr 7/18/2012 21:58'!
initialize
	<omniUnenforced>
	super initialize.
	immutabilityDomain := OcjImmutableDomain new.! !

!OcjAgentWithImmutability methodsFor: 'agent' stamp: 'StefanMarr 1/6/2012 15:00'!
processIncomingMessages
	super processIncomingMessages.
	immutabilityDomain adoptObject: state.! !

!Process methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/30/2007 12:09'!
currentTransaction
	^ currentTransaction! !

!Process methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/30/2007 12:09'!
currentTransaction: aTransaction
	currentTransaction := aTransaction! !

Error subclass: #LRSTMConflict
	instanceVariableNames: 'transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Errors'!

!LRSTMConflict methodsFor: 'accessing' stamp: 'lr 5/25/2007 09:04'!
transaction
	^ transaction! !

!LRSTMConflict methodsFor: 'accessing' stamp: 'lr 5/25/2007 09:04'!
transaction: aTransaction
	transaction := aTransaction! !

Error subclass: #LRSTMInvalidTransaction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Errors'!

!BlockContext methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 2/26/2008 18:16'!
atomic
	^ LRSTMTransaction new do: self! !

!BlockContext methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:13'!
atomicIfConflict: aBlock
	^ LRSTMTransaction new do: self ifConflict: aBlock! !

!BlockContext methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:14'!
atomicRetry
	^ LRSTMTransaction new retry: self! !

!ReadStream methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 15:16'!
atomicNext
	^ position >= readLimit
		ifFalse: [ collection at: (position := position + 1) ]! !

!BlockClosure methodsFor: '*OmniST-Languages-LRSTM' stamp: 'StefanMarr 11/28/2011 20:58'!
atomic
	^ LRSTMTransaction new do: self! !

!BlockClosure methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:14'!
atomicIfConflict: aBlock
	^ LRSTMTransaction new do: self ifConflict: aBlock! !

!BlockClosure methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/25/2007 09:15'!
atomicRetry
	^ LRSTMTransaction new retry: self! !

!SequenceableCollection methodsFor: '*OmniST-Languages-LRSTM' stamp: 'StefanMarr 7/29/2012 21:39'!
atomicReplaceFrom: start to: stop with: replacement startingAt: repStart
	"does not use the primitive to be track-able"
	| index repOff |
	repOff := repStart - start.
	index := start - 1.
	[ (index := index + 1) <= stop ]
		whileTrue: [ self at: index put: (replacement at: repOff + index) ]! !

Object subclass: #CSPiChannel
	instanceVariableNames: 'cell inDomain outDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-CSPi'!

!CSPiChannel class methodsFor: 'instance creation' stamp: 'StefanMarr 8/2/2012 12:52'!
in: domainUsingIn out: domainUsingOut
	| channel |
	channel := self new.
	channel instVarNamed: #inDomain  put: domainUsingIn.
	channel instVarNamed: #outDomain put: domainUsingOut.
	^ channel! !

!CSPiChannel methodsFor: 'channel ends' stamp: 'StefanMarr 8/2/2012 12:52'!
in
	^ CSPiChannelIn newWith: cell for: inDomain.! !

!CSPiChannel methodsFor: 'initialization' stamp: 'StefanMarr 1/9/2012 22:16'!
initialize
	cell := CSPiChannelCell new.! !

!CSPiChannel methodsFor: 'channel ends' stamp: 'StefanMarr 8/2/2012 12:52'!
out
	^ CSPiChannelOut newWith: cell for: outDomain.! !

Object subclass: #CSPiChannelCell
	instanceVariableNames: 'buffer sendSema receiveSema'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-CSPi-Reflection'!

!CSPiChannelCell methodsFor: 'initialization' stamp: 'StefanMarr 1/9/2012 22:11'!
initialize
	sendSema := Semaphore new.
	receiveSema := Semaphore new.! !

!CSPiChannelCell methodsFor: 'cell buffer' stamp: 'StefanMarr 1/9/2012 22:14'!
receive
	| result |
	sendSema wait.
	result := buffer.
	receiveSema signal.
	^ result! !

!CSPiChannelCell methodsFor: 'cell buffer' stamp: 'StefanMarr 1/9/2012 22:12'!
send: aValue
	buffer := aValue.
	sendSema signal.
	receiveSema wait.! !

Object subclass: #CSPiChannelIn
	instanceVariableNames: 'cell domain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-CSPi-Reflection'!

!CSPiChannelIn class methodsFor: 'instance creation' stamp: 'StefanMarr 8/2/2012 12:55'!
newWith: aCell for: domain
	| end |
	end := CSPiChannelIn new.
	end instVarNamed: #cell   put: aCell.
	end instVarNamed: #domain put: domain. 
	^ end.! !

!CSPiChannelIn methodsFor: 'message handling' stamp: 'StefanMarr 8/2/2012 13:07'!
send: aValue
	<omniUnenforced>
	cell send: aValue! !

Object subclass: #CSPiChannelOut
	instanceVariableNames: 'cell domain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-CSPi-Reflection'!

!CSPiChannelOut class methodsFor: 'instance creation' stamp: 'StefanMarr 8/2/2012 12:55'!
newWith: aCell for: domain
	| end |
	end := CSPiChannelOut new.
	end instVarNamed: #cell put: aCell.
	end instVarNamed: #domain put: domain.
	^ end! !

!CSPiChannelOut methodsFor: 'message handling' stamp: 'StefanMarr 8/2/2012 13:07'!
receive
	<omniUnenforced>
	| result |
	result := cell receive.
	domain adoptObject: result.
	^ result! !

Object subclass: #LRSTMChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMChange methodsFor: 'actions' stamp: 'lr 6/8/2007 16:59'!
apply
	self subclassResponsibility! !

!LRSTMChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasChanged
	^ true! !

!LRSTMChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasConflict
	^ false! !

LRSTMChange subclass: #LRSTMCustomChange
	instanceVariableNames: 'applyBlock conflictTestBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMCustomChange class methodsFor: 'instance-creation' stamp: 'StefanMarr 7/18/2012 22:02'!
onApply: anApplyBlock
	<omniUnenforced>
	^ self onApply: anApplyBlock onConflictTest: [ false ]! !

!LRSTMCustomChange class methodsFor: 'instance-creation' stamp: 'StefanMarr 7/18/2012 22:02'!
onApply: anApplyBlock onConflictTest: aConflictTestBlock
	<omniUnenforced>
	^ self basicNew initializeApply: anApplyBlock conflictTest: aConflictTestBlock! !

!LRSTMCustomChange methodsFor: 'actions' stamp: 'lr 2/26/2008 16:26'!
apply
	applyBlock value! !

!LRSTMCustomChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:28'!
hasConflict
	^ conflictTestBlock value! !

!LRSTMCustomChange methodsFor: 'initialization' stamp: 'lr 6/8/2007 17:04'!
initializeApply: anApplyBlock conflictTest: aConflictTestBlock	
	applyBlock := anApplyBlock.
	conflictTestBlock := aConflictTestBlock! !

LRSTMChange subclass: #LRSTMObjectChange
	instanceVariableNames: 'original previous working'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMObjectChange class methodsFor: 'instance-creation' stamp: 'lr 6/8/2007 16:59'!
on: anObject
	^ self basicNew initializeOn: anObject! !

!LRSTMObjectChange methodsFor: 'actions' stamp: 'lr 6/8/2007 16:59'!
apply
	original restoreSnapshot: working! !

!LRSTMObjectChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasChanged
	^ (working isIdenticalToSnapshot: previous) not! !

!LRSTMObjectChange methodsFor: 'testing' stamp: 'lr 6/8/2007 17:00'!
hasConflict
	^ (original isIdenticalToSnapshot: previous) not! !

!LRSTMObjectChange methodsFor: 'initialization' stamp: 'lr 6/8/2007 16:59'!
initializeOn: anObject
	original := anObject.
	working := original snapshotCopy.
	previous := working snapshotCopy! !

!LRSTMObjectChange methodsFor: 'accessing' stamp: 'lr 6/8/2007 17:00'!
original
	^ original! !

!LRSTMObjectChange methodsFor: 'accessing' stamp: 'lr 6/8/2007 17:00'!
previous
	^ previous! !

!LRSTMObjectChange methodsFor: 'printing' stamp: 'lr 6/8/2007 17:01'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' ['; print: self original; nextPut: $]! !

!LRSTMObjectChange methodsFor: 'accessing' stamp: 'lr 6/8/2007 17:00'!
working
	^ working! !

Object subclass: #LRSTMTransaction
	instanceVariableNames: 'parent context changes objectChanges'
	classVariableNames: ''
	poolDictionaries: 'ContextPart'
	category: 'OmniST-Languages-LRSTM-Model'!

!LRSTMTransaction class methodsFor: 'initialization' stamp: 'StefanMarr 12/11/2011 22:01'!
initialize
	super initialize.
	
	(Process hasInstVarNamed: #currentTransaction) ifFalse: [
		Process addInstVarNamed: #currentTransaction.
	].! !

!LRSTMTransaction class methodsFor: 'initialize-release' stamp: 'StefanMarr 12/11/2011 22:02'!
unload
	(Process hasInstVarNamed: #currentTransaction) ifTrue: [
		Process removeInstVarNamed: #currentTransaction.
	]! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 7/18/2012 21:55'!
abort
	"Abort a transaction."
	
	<omniUnenforced>
	parent := context := changes := objectChanges := nil! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 7/18/2012 21:56'!
abort: aValue
	"Abort a transaction."
	
	<omniUnenforced>
	Processor activeProcess currentTransaction: parent.
	thisContext swapSender: context.
	self abort.
	^ aValue! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 7/18/2012 21:56'!
addChange: aChange
	<omniUnenforced>
	^ changes add: aChange! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 7/18/2012 21:56'!
begin
	"Start a transaction. If this is a nested transaction copy the parent changes into our own context."
	
	<omniUnenforced>
	changes := OrderedCollection new.
	objectChanges := IdentityDictionary new.
	self isNested ifTrue: [
		parent objectChanges
			keysAndValuesDo: [ :key :value | objectChanges at: key put: value copy ] ]! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 7/18/2012 21:56'!
changeFor: anObject
	<omniUnenforced>
	^ objectChanges at: anObject ifAbsentPut: [ self addChange: (LRSTMObjectChange on: anObject) ]! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 7/18/2012 21:56'!
changes
	"Answer an ordered collection of unique changes."

	<omniUnenforced>
	^ changes! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 7/18/2012 21:56'!
checkpoint
	"Checkpoint a transaction."

	<omniUnenforced>
	self commit; begin! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 7/18/2012 21:56'!
commit
	"Commit a transaction."

	<omniUnenforced>
	self commitIfConflict: [ self signalConflict ]! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 7/18/2012 21:57'!
commitIfConflict: aBlock
	"Commit a transaction atomically."
	
	<omniUnenforced>
	[	changes do: [ :each |
			each hasConflict
				ifTrue: [ ^ aBlock value ] ].
		self isNested 
			ifTrue: [
				objectChanges keysAndValuesDo: [ :key :value |
					(parent objectChanges includesKey: key)
						ifFalse: [ parent addChange: value ].
					parent objectChanges at: key put: value ] ]
			ifFalse: [ 	
				changes do: [ :each |
					each hasChanged
						ifTrue: [ each apply ] ] ] ]
		valueUnpreemptively! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 7/18/2012 21:57'!
do: aBlock
	"Start a transaction with aBlock."

	<omniUnenforced>		
	^ self do: aBlock ifConflict: [ self signalConflict ]! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 8/2/2012 14:49'!
do: aBlock ifConflict: aConflictBlock
	"Evaluate aBLock within a new transaction, unless we are already in an existing transactional context. Evaluate aConflictBlock if the transaction conflicts with concurrent edits."

	<omniUnenforced>
	| result |
	parent := Processor activeProcess currentTransaction.
	Processor activeProcess currentTransaction: self.
	context := thisContext sender.
	self begin.
	
	"We need to use #evaluateInContext: here,
	 instead of #omni: to avoid undesired transformation"
	[ [result := LRSTMDomain domain evaluateInContextOnBaseLevel: aBlock ]
		ensure: [ Processor activeProcess currentTransaction: parent ]
	] ifCurtailed: [ self abort ].
 
	self commitIfConflict: aConflictBlock.
	^ result! !

!LRSTMTransaction methodsFor: 'testing' stamp: 'StefanMarr 7/18/2012 21:58'!
isNested
	<omniUnenforced>
	^ parent notNil! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 7/18/2012 22:01'!
objectChanges
	<omniUnenforced>
	^ objectChanges! !

!LRSTMTransaction methodsFor: 'accessing' stamp: 'StefanMarr 7/18/2012 22:02'!
parent
	<omniUnenforced>
	^ parent! !

!LRSTMTransaction methodsFor: 'public' stamp: 'StefanMarr 7/18/2012 22:08'!
retry: aBlock
	"Unconditionally retry to evaluate aBlock until there are no conflict."

	<omniUnenforced>
	^ self do: aBlock ifConflict: [ self retry: aBlock ]! !

!LRSTMTransaction methodsFor: 'private' stamp: 'StefanMarr 7/18/2012 22:08'!
signalConflict
	<omniUnenforced>
	^ LRSTMConflict new transaction: self; signal! !

Object subclass: #OPAtom
	instanceVariableNames: 'omniDomain state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure'!

!OPAtom class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 02:03'!
example
	| atom |
	atom := OPAtom new.

	Transcript log: 'New atom: ', atom read asString.

	atom reset: 1.

	[1 to: 10 do: [:i |
		atom swap: [:old | 
			Transcript log: '[atom-1] old: ', old asString, ' in iteration: ', i asString.
			Processor yield.
			old + 1]
	]] forkNamed: 'atom-1'.

	[1 to: 10 do: [:i |
		atom swap: [:old | 
			Transcript log: '[atom-2] old: ', old asString, ' in iteration: ', i asString.
			Processor yield.
			old + 1]
	]] forkNamed: 'atom-2'.
! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:43'!
compareTo: oldValue andSet: newValue
	"Clojure: (compare-and-set!! atom oldval newval)
	 Returns true on success, false otherwise."
	state = oldValue
		ifTrue:  [ state := newValue.
					 ^ true  ]
		ifFalse: [ ^ false ].! !

!OPAtom methodsFor: 'initialization' stamp: 'StefanMarr 11/6/2011 01:47'!
initialize
	omniDomain := OPAtomDomain newFor: self.! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:43'!
read
	"Clojure deref/@ reads the most recent value"
	^ state! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:45'!
reset: value
	state := value! !

!OPAtom methodsFor: 'atom' stamp: 'StefanMarr 11/6/2011 01:57'!
swap: updateBlock
	"Clojure: (swap!! atom f)
	 The block needs to take an argument,
	 and return a result value, like this:
	 [:oldVal | oldVal + 5]	"
	| currentState |
	[ | result |
	  currentState := state.
	  result := self compareTo: currentState
			andSet: (updateBlock value: currentState).
	  result ifFalse: [Transcript log: 'swap failed'].
	  result
		] whileFalse.
	
	! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 15:05'!
isIdenticalToSnapshot: anObject
	1 to: self class instSize do: [ :index | 
		(self instVarAt: index) == (anObject instVarAt: index)
			ifFalse: [ ^ false ] ].
	self class isVariable ifTrue: [
		1 to: self size do: [ :index | 
			(self at: index) == (anObject at: index)
				ifFalse: [ ^ false ] ] ].
	^ true! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 13:47'!
restoreSnapshot: anObject
	<primitive: 168>
	self primitiveFailed! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 6/7/2007 13:48'!
snapshotCopy
	<primitive: 148>
	self primitiveFailed! !

!Object methodsFor: '*OmniST-Languages-LRSTM' stamp: 'lr 5/30/2007 12:11'!
workingCopy
	"Answer a working copy to be used within the context of the current transaction."
	
	| transaction |
	transaction := Processor activeProcess currentTransaction
		ifNil: [ self error: 'No active transaction' ].
	^ (transaction changeFor: self) working! !

Object subclass: #OcjAgentExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-Clojure'!

!OcjAgentExample class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/7/2012 12:40'!
example
	| agent |
	agent := OcjAgent new: 1.

	[ 1 to: 10 do: [:i|
		agent send: [:old |
			Transcript log: '[agent-1] old: ', old asString, ' iteration: ', i asString.
			old + 1].
		(Delay forSeconds: 1) wait.
	]] forkNamed: 'agent-1'.

	[ 1 to: 10 do: [:i|
		agent send: [:old |
			Transcript log: '[agent-2] old: ', old asString, ' iteration: ', i asString.
			old + 1].
		(Delay forMilliseconds: 800) wait.
	]] forkNamed: 'agent-2'.

	[ 1 to: 20 do: [:i|
		agent send: [:old |
			Transcript log: '[agent-3] old: ', old asString, ' iteration: ', i asString.
			old + 1].
		(Delay forMilliseconds: 512) wait.
	]] forkNamed: 'agent-3'.

	Transcript log: 'Started all agent tester processes.'.! !

Object subclass: #OstExperiments
	instanceVariableNames: 'omniDomain a b c d'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Languages-POCs'!

!OstExperiments commentStamp: '<historical>' prior: 0!
This class had only a use while developing the compiler transformations.!

!OstExperiments methodsFor: 'nil' stamp: 'StefanMarr 12/7/2011 23:19'!
a: val
	a := val! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 23:20'!
a: anA b: aB andSome: aC
	a := anA + aB + aC! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 22:22'!
aAndB
	^ a + b! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 23:20'!
abc
 | sd |
 a ifNil: [^b].
 a == b ifTrue: [^ a].
 ^ c! !

!OstExperiments methodsFor: 'nil' stamp: 'StefanMarr 12/8/2011 00:33'!
do: p1 a: p2 b: p3 c: p4 d: p5 e: p6
	^ p1 asString,
	  p2 asString,
	  p3 asString,
	  p4 asString,
	  p5 asString,
	  p6 asString
	  ! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/22/2012 16:21'!
manualTestChainedSend
	| result |
	
	result := [ 
		| x_omni_1_ |
		x_omni_1_ := 1 omniDomain requestExecution: 2 of: #bitOr: on: 1.
		x_omni_1_ omniDomain requestExecution: 4 of: #bitOr: on: x_omni_1_
	] fooBarEntryMessage.

	self omniDomain
		requestExecution: 7
		with: result
		of: #xx_omni_x_assert:equals:
		on: self! !

!OstExperiments methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/8/2011 00:31'!
perform: sel a: p1 b: p2 c: p3 d: p4 e: p5 f: p6
	"dd"
	<primitive: 83>
	^ -1! !
LRSTMTransaction initialize!
LRSTMDomain initialize!
